searchNodes=[{"doc":"","ref":"Mix.Tasks.Still.Compile.html","title":"Mix.Tasks.Still.Compile","type":"task"},{"doc":"","ref":"Mix.Tasks.Still.Dev.html","title":"Mix.Tasks.Still.Dev","type":"task"},{"doc":"Defines functions to be used by the several preprocessors as well as the behaviour they should have.Preprocessors are the cornerstone of Still. A preprocessor chain can take a markdown file, execute its embedded Elixir, extract metadata from its front matter, transform it into HTML and wrap it in a layout.The default preprocessor chain is the following:%{ &quot;.slim&quot; =&gt; [AddContent, EEx, Frontmatter, Slime, OutputPath, AddLayout, Save], &quot;.slime&quot; =&gt; [AddContent, EEx, Frontmatter, Slime, OutputPath, AddLayout, Save], &quot;.eex&quot; =&gt; [AddContent, EEx, Frontmatter, OutputPath, AddLayout, Save], &quot;.css&quot; =&gt; [AddContent, EEx, CSSMinify, OutputPath, URLFingerprinting, AddLayout, Save], &quot;.js&quot; =&gt; [AddContent, EEx, JS, OutputPath, URLFingerprinting, AddLayout, Save], &quot;.md&quot; =&gt; [AddContent, EEx, Frontmatter, Markdown, OutputPath, AddLayout, Save], &quot;.jpg&quot; =&gt; [OutputPath, Image], &quot;.png&quot; =&gt; [OutputPath, Image] }If the default preprocessors are not enough, you can extend Still with your own.A custom preprocessor is simply a module that calls use Still.Preprocessor and implements the render/2 and extension/1 functions.Take the following example:defmodule YourSite.JPEG do use Still.Preprocessor @impl true def extension(_), do: &quot;.jpeg&quot; @impl true def render(file) do file end endIn this example, the render/1 function is used to transform the content and the metadata of a file, and the extension/1 function is used to set the resulting content type. This extension/1 function is not mandatory.See the preprocessor guide for more details.","ref":"Still.Preprocessor.html","title":"Still.Preprocessor","type":"behaviour"},{"doc":"","ref":"Still.Preprocessor.html#c:extension/1","title":"Still.Preprocessor.extension/1","type":"callback"},{"doc":"Retrieves the preprocessor pipeline for the given file.","ref":"Still.Preprocessor.html#for/1","title":"Still.Preprocessor.for/1","type":"function"},{"doc":"","ref":"Still.Preprocessor.html#c:render/1","title":"Still.Preprocessor.render/1","type":"callback"},{"doc":"Runs the preprocessor pipeline for the given file.","ref":"Still.Preprocessor.html#run/1","title":"Still.Preprocessor.run/1","type":"function"},{"doc":"","ref":"Still.Preprocessor.html#run/2","title":"Still.Preprocessor.run/2","type":"function"},{"doc":"Implements a profiler that keeps track of the time each file takes to render.A file can be rendered or compiled multiple times in the same stage (e.g when being included), so the compiler hashes each file and metadata to distinguish between those.After the compilation is finished, a profiler report is generated and can be accessed at /profiler.html.The profiler should only run in development and can be disabled by setting:config :still, profiler: false","ref":"Still.Profiler.html","title":"Still.Profiler","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Still.Profiler.html#child_spec/1","title":"Still.Profiler.child_spec/1","type":"function"},{"doc":"Recompiles the HTML page. Should only be used for internal maintenance.","ref":"Still.Profiler.html#recompile/0","title":"Still.Profiler.recompile/0","type":"function"},{"doc":"Save a Still.SourceFile and the rendering/compilation delta.","ref":"Still.Profiler.html#register/2","title":"Still.Profiler.register/2","type":"function"},{"doc":"Return a timestamp of the current system time in millseconds.","ref":"Still.Profiler.html#timestamp/0","title":"Still.Profiler.timestamp/0","type":"function"},{"doc":"The SourceFile retains all information regarding a file being compiled in the system. It's conceptually similar to the conn variable in a Phoenix app. The idea is that different subsystems transform this struct and pass it to the next.","ref":"Still.SourceFile.html","title":"Still.SourceFile","type":"module"},{"doc":"","ref":"Still.SourceFile.html#t:t/0","title":"Still.SourceFile.t/0","type":"type"},{"doc":"Collection of utility functions.","ref":"Still.Utils.html","title":"Still.Utils","type":"module"},{"doc":"Recursively removes all files from the site's output directory.","ref":"Still.Utils.html#clean_output_dir/0","title":"Still.Utils.clean_output_dir/0","type":"function"},{"doc":"Recursively removes all files from the given path, relative to the output directory.","ref":"Still.Utils.html#clean_output_dir/1","title":"Still.Utils.clean_output_dir/1","type":"function"},{"doc":"Returns the value configured for :still by the given key. Returns the provided default if it doesn't exist.","ref":"Still.Utils.html#config/2","title":"Still.Utils.config/2","type":"function"},{"doc":"Returns the value configured for :still by the given key. Errors if it doesn't exist.","ref":"Still.Utils.html#config!/1","title":"Still.Utils.config!/1","type":"function"},{"doc":"Returns the site's base URL.","ref":"Still.Utils.html#get_base_url/0","title":"Still.Utils.get_base_url/0","type":"function"},{"doc":"Delegates the call to the current Still.Preprocessor.Image.Adapter.","ref":"Still.Utils.html#get_image_info/1","title":"Still.Utils.get_image_info/1","type":"function"},{"doc":"Returns the absolute path configured as the site's entrypoint.This is the value set byconfig :still, input: &quot;path/to/site&quot;","ref":"Still.Utils.html#get_input_path/0","title":"Still.Utils.get_input_path/0","type":"function"},{"doc":"Returns the current input path for a given file, prepending it with the site's input directory. See get_input_path/0.","ref":"Still.Utils.html#get_input_path/1","title":"Still.Utils.get_input_path/1","type":"function"},{"doc":"Returns the modified time of a given file as a DateTime struct or the :error atom if the file doesn't exist.","ref":"Still.Utils.html#get_modified_time/1","title":"Still.Utils.get_modified_time/1","type":"function"},{"doc":"Returns the modified time of a given file. Errors if the file does not exist.","ref":"Still.Utils.html#get_modified_time!/1","title":"Still.Utils.get_modified_time!/1","type":"function"},{"doc":"Returns the absolute path configured as the site's output destination.This is the value set byconfig :still, output: &quot;path/to/site&quot;","ref":"Still.Utils.html#get_output_path/0","title":"Still.Utils.get_output_path/0","type":"function"},{"doc":"Returns the current output path for a given file, prepending it with the site's output directory. See get_output_path/0.","ref":"Still.Utils.html#get_output_path/1","title":"Still.Utils.get_output_path/1","type":"function"},{"doc":"Receives an absolute path and converts it to relative by trimming the site's entrypoint directory.","ref":"Still.Utils.html#get_relative_input_path/1","title":"Still.Utils.get_relative_input_path/1","type":"function"},{"doc":"Creates the output directory.","ref":"Still.Utils.html#mk_output_dir/0","title":"Still.Utils.mk_output_dir/0","type":"function"},{"doc":"Creates the directory by the given path, relative to the output directory.","ref":"Still.Utils.html#mk_output_dir/1","title":"Still.Utils.mk_output_dir/1","type":"function"},{"doc":"Recursively cleans the site's output directory.","ref":"Still.Utils.html#rm_output_dir/0","title":"Still.Utils.rm_output_dir/0","type":"function"},{"doc":"Collection of utility functions specific for extending the behaviour of map structures.","ref":"Still.Utils.Map.html","title":"Still.Utils.Map","type":"module"},{"doc":"Converts all string keys of a map into atoms. Traverses the map to ensure that any subkeys are also converted.","ref":"Still.Utils.Map.html#deep_atomify_keys/1","title":"Still.Utils.Map.deep_atomify_keys/1","type":"function"},{"doc":"File system watcher that triggers compilation for new files, recompilation for changed files and kills ane Still.Compiler.Incremental.Node for removed files. Should only be used in the dev environment.","ref":"Still.Watcher.html","title":"Still.Watcher","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Still.Watcher.html#child_spec/1","title":"Still.Watcher.child_spec/1","type":"function"},{"doc":"Starts the file system watcher and the compiler traversal via Still.Compiler.Traverse.","ref":"Still.Watcher.html#handle_continue/2","title":"Still.Watcher.handle_continue/2","type":"function"},{"doc":"Handles file events, triggering compilation for new files, recompilation for modifications and removal of Still.Compiler.Incremental.Node for deletions.","ref":"Still.Watcher.html#handle_info/2","title":"Still.Watcher.handle_info/2","type":"function"},{"doc":"","ref":"Still.Watcher.html#start_link/1","title":"Still.Watcher.start_link/1","type":"function"},{"doc":"Keeps track of all the collections within a compilation cycle.","ref":"Still.Compiler.Collections.html","title":"Still.Compiler.Collections","type":"module"},{"doc":"Adds a file to its collections.","ref":"Still.Compiler.Collections.html#add/1","title":"Still.Compiler.Collections.add/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Still.Compiler.Collections.html#child_spec/1","title":"Still.Compiler.Collections.child_spec/1","type":"function"},{"doc":"Returns the collection with a given name, subscribing caller's file to any future changes.","ref":"Still.Compiler.Collections.html#get/2","title":"Still.Compiler.Collections.get/2","type":"function"},{"doc":"Resets the saved collections.","ref":"Still.Compiler.Collections.html#reset/0","title":"Still.Compiler.Collections.reset/0","type":"function"},{"doc":"","ref":"Still.Compiler.Collections.html#start_link/1","title":"Still.Compiler.Collections.start_link/1","type":"function"},{"doc":"Almost every compilation request goes through CompilationStage. This process is responsible for keeping track of subscriptions (e.g: a browser subscribing to changes) and notifying all the subscribers of the end of the compilation cycle.Subscribers to this process are notified when the queue is empty, which is usefull to refresh the browser or finish the compilation task in production.Subscribers receive the event :bus_empty when CompilationStage's compilation cycle is finished.There are many events that lead to a file being compiled:when Still starts, all files are compiled;files that change are compiled;files that include files that have changed are compiled;any many more.","ref":"Still.Compiler.CompilationStage.html","title":"Still.Compiler.CompilationStage","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Still.Compiler.CompilationStage.html#child_spec/1","title":"Still.Compiler.CompilationStage.child_spec/1","type":"function"},{"doc":"Asynchronously saves a file in the compilation list.Files are compiled in parallel, meaning that every 100ms the compilation stage will run and compile any due source file. When no more files are ready to be compiled, the subscribers are notified.","ref":"Still.Compiler.CompilationStage.html#compile/1","title":"Still.Compiler.CompilationStage.compile/1","type":"function"},{"doc":"","ref":"Still.Compiler.CompilationStage.html#start_link/1","title":"Still.Compiler.CompilationStage.start_link/1","type":"function"},{"doc":"Save a subscription to the compilation cycle.","ref":"Still.Compiler.CompilationStage.html#subscribe/0","title":"Still.Compiler.CompilationStage.subscribe/0","type":"function"},{"doc":"Remove a subscription to the compilation cycle.","ref":"Still.Compiler.CompilationStage.html#unsubscribe/0","title":"Still.Compiler.CompilationStage.unsubscribe/0","type":"function"},{"doc":"Saves an error occurring within a file's compilation and allows them to be retrieved for a prettified display.Since files are compiled asynchronously, the browser (or other interested parties) require a centralised access to compilation errors.","ref":"Still.Compiler.ErrorCache.html","title":"Still.Compiler.ErrorCache","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Still.Compiler.ErrorCache.html#child_spec/1","title":"Still.Compiler.ErrorCache.child_spec/1","type":"function"},{"doc":"Retrieve all saved errors, for all files.","ref":"Still.Compiler.ErrorCache.html#get_errors/0","title":"Still.Compiler.ErrorCache.get_errors/0","type":"function"},{"doc":"Save the given error or clear it if the compilation was successful.If result is {:ok, source_file}, the error cache for the given file is cleared. Otherwise it is updated.","ref":"Still.Compiler.ErrorCache.html#set/1","title":"Still.Compiler.ErrorCache.set/1","type":"function"},{"doc":"","ref":"Still.Compiler.ErrorCache.html#start_link/1","title":"Still.Compiler.ErrorCache.start_link/1","type":"function"},{"doc":"Responsible for compiling or rendering a given file.The difference between compilation and renderisation is that the former outputs the source file's contents, after being processed, to the correct destination, while the latter only runs the content through the preprocessor pipeline.","ref":"Still.Compiler.File.html","title":"Still.Compiler.File","type":"module"},{"doc":"Compiles a given Still.SourceFile to the correct output path, after being run through its Still.Preprocessor pipeline.","ref":"Still.Compiler.File.html#compile/1","title":"Still.Compiler.File.compile/1","type":"function"},{"doc":"Renders a given Still.SourceFile, using the correct metadata.This differs from compilation since it doesn't generate any output in the file system.","ref":"Still.Compiler.File.html#render/2","title":"Still.Compiler.File.render/2","type":"function"},{"doc":"Responsible for wrapping a portion of markup in a pre-defined development layout and compiling the output. Should only be used in a dev environment.The development layout used is in priv/still/dev.slime and handles the styling of errors and subscribing to file updates, causing the browser to refresh.","ref":"Still.Compiler.File.DevLayout.html","title":"Still.Compiler.File.DevLayout","type":"module"},{"doc":"Wraps the given content in a Still.SourceFile containing the development layout as input and the content as children.","ref":"Still.Compiler.File.DevLayout.html#wrap/1","title":"Still.Compiler.File.DevLayout.wrap/1","type":"function"},{"doc":"An incremental node represents a file, or folder, that is processed individually.Each file has a list of subscriptions and subcribers. The subscriptions are the files included by the current file. The subscribers are the files that the current file includes. When the current file changes, it notifies the subscribers and updates the subscriptions.A file can be compiled or rendered:Compile - compiling a file means, most times, running it thorugh a preprocessor and writing to to the destination folder.Render - rendering a file means that the current file is being included by another file. Template files may return HTML and images could return a path.Incremental nodes attempt to compile/render files synchronously. This process can take a long time, which is usually fine, but it can be changed by setting the :compilation_timeout key in your config/config.exs. Default is :infinity.","ref":"Still.Compiler.Incremental.Node.html","title":"Still.Compiler.Incremental.Node","type":"module"},{"doc":"Adds a file to the list of files this process is subscribed to.","ref":"Still.Compiler.Incremental.Node.html#add_subscription/2","title":"Still.Compiler.Incremental.Node.add_subscription/2","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Still.Compiler.Incremental.Node.html#child_spec/1","title":"Still.Compiler.Incremental.Node.child_spec/1","type":"function"},{"doc":"Returns the compilation timeout defined in the config.You can change this by setting config :still, :compilation_timeout, 1_000_000","ref":"Still.Compiler.Incremental.Node.html#compilation_timeout/0","title":"Still.Compiler.Incremental.Node.compilation_timeout/0","type":"function"},{"doc":"Compiles the file mapped by the Node with the given PID.This PID can be obtained from Still.Compiler.Incremental.Registry.For difference between compilation and renderisation see Still.Compiler.File.","ref":"Still.Compiler.Incremental.Node.html#compile/1","title":"Still.Compiler.Incremental.Node.compile/1","type":"function"},{"doc":"Removes a file from the list of files subscribing to this process.","ref":"Still.Compiler.Incremental.Node.html#remove_subscriber/2","title":"Still.Compiler.Incremental.Node.remove_subscriber/2","type":"function"},{"doc":"Renders the file mapped by the Node with the given PID.This PID can be obtained from Still.Compiler.Incremental.Registry.For difference between compilation and renderisation see Still.Compiler.File.","ref":"Still.Compiler.Incremental.Node.html#render/3","title":"Still.Compiler.Incremental.Node.render/3","type":"function"},{"doc":"","ref":"Still.Compiler.Incremental.Node.html#start_link/1","title":"Still.Compiler.Incremental.Node.start_link/1","type":"function"},{"doc":"Compiles the contents of a Still.Compiler.Incremental.Node.First attempts a pass through copy of the file, in case it should be ignored. When this isn't successful, attempts a compilation via Still.Compiler.File, notifying any relevant subscribers of changes.","ref":"Still.Compiler.Incremental.Node.Compile.html","title":"Still.Compiler.Incremental.Node.Compile","type":"module"},{"doc":"","ref":"Still.Compiler.Incremental.Node.Compile.html#run/1","title":"Still.Compiler.Incremental.Node.Compile.run/1","type":"function"},{"doc":"Supervisor that maps files (based on their name) to PIDs of Still.Compiler.Incremental.Node.","ref":"Still.Compiler.Incremental.Registry.html","title":"Still.Compiler.Incremental.Registry","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Still.Compiler.Incremental.Registry.html#child_spec/1","title":"Still.Compiler.Incremental.Registry.child_spec/1","type":"function"},{"doc":"Attempts to retrive the Still.Compiler.Incremental.Node corresponding to the given file name. If none exists, a new node is started.","ref":"Still.Compiler.Incremental.Registry.html#get_or_create_file_process/1","title":"Still.Compiler.Incremental.Registry.get_or_create_file_process/1","type":"function"},{"doc":"","ref":"Still.Compiler.Incremental.Registry.html#start_link/1","title":"Still.Compiler.Incremental.Registry.start_link/1","type":"function"},{"doc":"Terminates the Still.Compiler.Incremental.Node corresponding to the given file name.","ref":"Still.Compiler.Incremental.Registry.html#terminate_file_process/1","title":"Still.Compiler.Incremental.Registry.terminate_file_process/1","type":"function"},{"doc":"Copies a file from the input path to the output directory without changing it.Matching parametersYou can configure matching parameters by settingconfig :still, pass_through_copy: [&quot;img/logo.png&quot;]In the example above, the file logo.png inside the img folder will be copied to the img folder in the output. But if you write something like this:config :still, pass_through_copy: [&quot;img&quot;]Any file or folder that starts with the string img will be copied, which may include an img folder or a file named img.png. So you need to be mindful of that.You can also use regular expressions:config still, pass_through_copy: [~r/.*.jpe?g/]Sometimes you want to alter the file name or path but keep the content of the files. The configuration allows this by using tuples. The key will be used to match the input folder, and the value will be used to transform the input path:config :still, pass_through_copy: [css: &quot;styles&quot;] # this is also valid: # config :still, # pass_through_copy: [{&quot;css&quot;, &quot;styles&quot;}]In the example above, the css folder from the input folder but will be renamed to styles in the output folder.","ref":"Still.Compiler.PassThroughCopy.html","title":"Still.Compiler.PassThroughCopy","type":"module"},{"doc":"Attempts to copy a file from the input path to the output directory without changing.If the file doesn't match any configured name, :no_match is returned.See the Matching Parameters section.","ref":"Still.Compiler.PassThroughCopy.html#try/1","title":"Still.Compiler.PassThroughCopy.try/1","type":"function"},{"doc":"First step in the compilation stage. Traverses the input directory, adding any files or folders to Still.Compiler.Incremental.Registry.","ref":"Still.Compiler.Traverse.html","title":"Still.Compiler.Traverse","type":"module"},{"doc":"","ref":"Still.Compiler.Traverse.html#run/0","title":"Still.Compiler.Traverse.run/0","type":"function"},{"doc":"Set of helper functions to be included in a file that runs through an Elixir preprocessor.","ref":"Still.Compiler.ViewHelpers.html","title":"Still.Compiler.ViewHelpers","type":"module"},{"doc":"Implements an arbitrary content tag with the given content.","ref":"Still.Compiler.ViewHelpers.ContentTag.html","title":"Still.Compiler.ViewHelpers.ContentTag","type":"module"},{"doc":"Renders an arbitrary HTML tag with the given content.If content is nil, the rendered tag is self-closing.opts should contain the relevant HTML attributes (e.g class: &quot;myelem&quot;).aria attributes should be in the aria_name format (e.g: aria_label: &quot;Label&quot;).All data attributes should be within a data array (e.g: data: [method: &quot;POST&quot;, foo: &quot;bar&quot;]).Examplesiex&gt; content_tag(&quot;a&quot;, &quot;My link&quot;, href: &quot;https://example.org&quot;, data: [method: &quot;POST&quot;, something: &quot;value&quot;], aria_label: &quot;Label&quot;) &quot;&lt;a href=\\&quot;https://example.org\\&quot; data-method=\\&quot;POST\\&quot; data-something=\\&quot;value\\&quot; aria-label=\\&quot;Label\\&quot;&gt;My link&lt;/a&gt;&quot;","ref":"Still.Compiler.ViewHelpers.ContentTag.html#render/3","title":"Still.Compiler.ViewHelpers.ContentTag.render/3","type":"function"},{"doc":"Renders an anchor HTML tag.","ref":"Still.Compiler.ViewHelpers.Link.html","title":"Still.Compiler.ViewHelpers.Link","type":"module"},{"doc":"Uses Still.Compiler.ViewHelpers.ContentTag to render an anchor tag.Requires a :to option, the target URL. If this is a relative path, the website's base URL is prepended. If it is an absolute path the target: &quot;_blank&quot; and rel: &quot;noopener noreferrer&quot; options are added to be passed to Still.Compiler.ViewHelpers.ContentTag render function.If there is a do block, it uses the current file preprocessor to render markup. Note that this is on demand, outside Still.Compiler.CompilationStage.","ref":"Still.Compiler.ViewHelpers.Link.html#render/3","title":"Still.Compiler.ViewHelpers.Link.render/3","type":"function"},{"doc":"Generates a link HTML tag to the target CSS file.This file must exist and to ensure that, it will be compiled outside Still.Compiler.CompilationStage.","ref":"Still.Compiler.ViewHelpers.LinkToCSS.html","title":"Still.Compiler.ViewHelpers.LinkToCSS","type":"module"},{"doc":"Generates a link HTML tag to the target CSS file.All options are converted to the attr=value format.This file must exist and to ensure that, it will be compiled outside Still.Compiler.CompilationStage.","ref":"Still.Compiler.ViewHelpers.LinkToCSS.html#render/2","title":"Still.Compiler.ViewHelpers.LinkToCSS.render/2","type":"function"},{"doc":"Generates a script HTML tag to the target JS file.This file must exist and to ensure that, it will be compiled outside Still.Compiler.CompilationStage.","ref":"Still.Compiler.ViewHelpers.LinkToJS.html","title":"Still.Compiler.ViewHelpers.LinkToJS","type":"module"},{"doc":"Generates a script HTML tag to the target JS file.All options are converted to the attr=value format.This file must exist and to ensure that, it will be compiled outside Still.Compiler.CompilationStage.","ref":"Still.Compiler.ViewHelpers.LinkToJS.html#render/2","title":"Still.Compiler.ViewHelpers.LinkToJS.render/2","type":"function"},{"doc":"Generates a set of images to ensure they are responsive.See Still.Preprocessor.Image for details on these transformations.","ref":"Still.Compiler.ViewHelpers.ResponsiveImage.html","title":"Still.Compiler.ViewHelpers.ResponsiveImage","type":"module"},{"doc":"Returns an image tag with the src and srcset.If :sizes or :transformations are present in opts, they will be passed to Still.Preprocessor.Image.If :sizes is not set, the default will be 25%, 50%, 75% and 100% of the input file's width.","ref":"Still.Compiler.ViewHelpers.ResponsiveImage.html#render/2","title":"Still.Compiler.ViewHelpers.ResponsiveImage.render/2","type":"function"},{"doc":"Renders the given content as safe HTML, escaping any tags.","ref":"Still.Compiler.ViewHelpers.SafeHTML.html","title":"Still.Compiler.ViewHelpers.SafeHTML","type":"module"},{"doc":"","ref":"Still.Compiler.ViewHelpers.SafeHTML.html#render/1","title":"Still.Compiler.ViewHelpers.SafeHTML.render/1","type":"function"},{"doc":"Truncates the content to a given maximum.Optionslength - Maximum length of the truncated string. This includes the omission string. Defaults to 300.omission - String to indicate the omission of further text. Defaults to &quot;...&quot;","ref":"Still.Compiler.ViewHelpers.Truncate.html","title":"Still.Compiler.ViewHelpers.Truncate","type":"module"},{"doc":"","ref":"Still.Compiler.ViewHelpers.Truncate.html#render/2","title":"Still.Compiler.ViewHelpers.Truncate.render/2","type":"function"},{"doc":"Converts a relative path to an absolute path.","ref":"Still.Compiler.ViewHelpers.UrlFor.html","title":"Still.Compiler.ViewHelpers.UrlFor","type":"module"},{"doc":"Converts the given relative path to an absolute path by prepending the application's base URL and removing references to index.html.","ref":"Still.Compiler.ViewHelpers.UrlFor.html#render/1","title":"Still.Compiler.ViewHelpers.UrlFor.render/1","type":"function"},{"doc":"Still.Preprocessor that reads the content of the input file and sets it to the :content field of Still.SourceFile.","ref":"Still.Preprocessor.AddContent.html","title":"Still.Preprocessor.AddContent","type":"module"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.AddContent.html#extension/1","title":"Still.Preprocessor.AddContent.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.AddContent.html#run/1","title":"Still.Preprocessor.AddContent.run/1","type":"function"},{"doc":"Still.Preprocessor that renders the layout of a given file and wraps it around the content of that same file.Note that this rendering happens outside Still.Compiler.CompilationStage.For this preprocessor to work, it requires a :layout key in the metadata, which can be set in the frontmatter and then added to the metadata by Still.Preprocessor.Frontmatter.","ref":"Still.Preprocessor.AddLayout.html","title":"Still.Preprocessor.AddLayout","type":"module"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.AddLayout.html#extension/1","title":"Still.Preprocessor.AddLayout.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.AddLayout.html#run/1","title":"Still.Preprocessor.AddLayout.run/1","type":"function"},{"doc":"Minifies a CSS file. This is a very basic minifier that simply removes whitespaces.","ref":"Still.Preprocessor.CSSMinify.html","title":"Still.Preprocessor.CSSMinify","type":"module"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.CSSMinify.html#extension/1","title":"Still.Preprocessor.CSSMinify.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.CSSMinify.html#run/1","title":"Still.Preprocessor.CSSMinify.run/1","type":"function"},{"doc":"Renders an EEx file. See Still.Preprocessor.Renderer and Still.Preprocessor.EEx.Renderer.","ref":"Still.Preprocessor.EEx.html","title":"Still.Preprocessor.EEx","type":"module"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.EEx.html#extension/1","title":"Still.Preprocessor.EEx.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.EEx.html#run/1","title":"Still.Preprocessor.EEx.run/1","type":"function"},{"doc":"Still.Preprocessor.Renderer implementation for EEx files.","ref":"Still.Preprocessor.EEx.Renderer.html","title":"Still.Preprocessor.EEx.Renderer","type":"module"},{"doc":"","ref":"Still.Preprocessor.EEx.Renderer.html#create/1","title":"Still.Preprocessor.EEx.Renderer.create/1","type":"function"},{"doc":"Parses the file's frontmatter and adds they key/value pairs to the corresponding Still.SourceFile's :metadata field.","ref":"Still.Preprocessor.Frontmatter.html","title":"Still.Preprocessor.Frontmatter","type":"module"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.Frontmatter.html#extension/1","title":"Still.Preprocessor.Frontmatter.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.Frontmatter.html#run/1","title":"Still.Preprocessor.Frontmatter.run/1","type":"function"},{"doc":"Handles image transformation.To configure it, set :image_opts in the metadata:%{ image_opts: %{ sizes: [100, 200], transformations: [color_filter: &quot;grayscale_bt709&quot;] } }:sizes defines the widths of the output files to create.:transformations defines the function name and arguments to call on the adapter. By default, the adapter is Still.Preprocessor.Image.Mogrify. However you can also include your own or make use of Still.Preprocessor.Image.Imageflow by adding :still_imageflow as a dependency and setting in your config: config :still, :image_adapter, Still.Preprocessor.Image.ImageflowFor more information see Mogrify's options or ImageMagick's docs information.When :image_opts is not set, it copies the input file to the output file as it is.","ref":"Still.Preprocessor.Image.html","title":"Still.Preprocessor.Image","type":"module"},{"doc":"","ref":"Still.Preprocessor.Image.html#adapter/0","title":"Still.Preprocessor.Image.adapter/0","type":"function"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.Image.html#extension/1","title":"Still.Preprocessor.Image.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.Image.html#run/1","title":"Still.Preprocessor.Image.run/1","type":"function"},{"doc":"Adapter definition to be used by Still.Preprocessor.Image.An adapter implements the render/1 function. It is only called when :image_opts are defined within the :metadata field of the source file (see Still.SourceFile).The render/1 function should create several versions of the same images, given the :sizes and :transformations options. The purpose of this function is to support the responsive_image/2 view helper. See Still.Compiler.ViewHelpers.ResponsiveImage.render/2 for details on how this view helper works.:sizes defines the widths of the output files to create.:transformations defines the function name and arguments to call on the adapter. By default, the adapter is Still.Preprocessor.Image.Mogrify.You can change the adapter by altering your config: config :still, :image_adapter, Still.Preprocessor.Image.ImageflowThe Imageflow adapter requires you to have :still_imageflow as a dependency.","ref":"Still.Preprocessor.Image.Adapter.html","title":"Still.Preprocessor.Image.Adapter","type":"behaviour"},{"doc":"","ref":"Still.Preprocessor.Image.Adapter.html#c:get_image_info/1","title":"Still.Preprocessor.Image.Adapter.get_image_info/1","type":"callback"},{"doc":"","ref":"Still.Preprocessor.Image.Adapter.html#c:render/1","title":"Still.Preprocessor.Image.Adapter.render/1","type":"callback"},{"doc":"Implements Still.Preprocessor.Image.Adapter for Mogrify.Default module used when no other adapter is provided.","ref":"Still.Preprocessor.Image.Mogrify.html","title":"Still.Preprocessor.Image.Mogrify","type":"module"},{"doc":"Preprocessor for JavaScript files that simply bypasses the file's contents.","ref":"Still.Preprocessor.JS.html","title":"Still.Preprocessor.JS","type":"module"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.JS.html#extension/1","title":"Still.Preprocessor.JS.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.JS.html#run/1","title":"Still.Preprocessor.JS.run/1","type":"function"},{"doc":"Renders markdown files using Markdown.","ref":"Still.Preprocessor.Markdown.html","title":"Still.Preprocessor.Markdown","type":"module"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.Markdown.html#extension/1","title":"Still.Preprocessor.Markdown.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.Markdown.html#run/1","title":"Still.Preprocessor.Markdown.run/1","type":"function"},{"doc":"Generates the output path based on the Still.SourceFile :input_path and :extension field, adding it to the :output_file field.","ref":"Still.Preprocessor.OutputPath.html","title":"Still.Preprocessor.OutputPath","type":"module"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.OutputPath.html#extension/1","title":"Still.Preprocessor.OutputPath.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.OutputPath.html#run/1","title":"Still.Preprocessor.OutputPath.run/1","type":"function"},{"doc":"Defines the basic attributes of a markup renderer.A renderer needs to implement a compile/2 function and an optional ast/0 function. When a markup file is being compiled, a module is created on demand. This module imports all view helpers defined by Still as well as any view helper configured by the user:config :still, view_helpers: [Your.Module]The created module implements a render/0 which will return the result of the compile/2 call.The ast/0 can be used to tap into the AST of the new module and import or require any necessary module.Markup renderers should use Still.Preprocessor.Renderer and provide two options::extensions - the list of extensions compiled by the renderer;:preprocessor - the preprocessor used to render any necessary snippets (e.g via Still.Compiler.ViewHelpers.ContentTag).","ref":"Still.Preprocessor.Renderer.html","title":"Still.Preprocessor.Renderer","type":"behaviour"},{"doc":"","ref":"Still.Preprocessor.Renderer.html#c:ast/0","title":"Still.Preprocessor.Renderer.ast/0","type":"callback"},{"doc":"","ref":"Still.Preprocessor.Renderer.html#c:compile/2","title":"Still.Preprocessor.Renderer.compile/2","type":"callback"},{"doc":"","ref":"Still.Preprocessor.Renderer.html#t:ast/0","title":"Still.Preprocessor.Renderer.ast/0","type":"type"},{"doc":"Writes the Still.SourceFile :content field to the :output_file field, creating any necessary directories and including the development layout if needed.See Still.Compiler.File.DevLayout.","ref":"Still.Preprocessor.Save.html","title":"Still.Preprocessor.Save","type":"module"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.Save.html#extension/1","title":"Still.Preprocessor.Save.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.Save.html#run/1","title":"Still.Preprocessor.Save.run/1","type":"function"},{"doc":"Renders a Slime file. See Still.Preprocessor.Renderer and Still.Preprocessor.EEx.Renderer.","ref":"Still.Preprocessor.Slime.html","title":"Still.Preprocessor.Slime","type":"module"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.Slime.html#extension/1","title":"Still.Preprocessor.Slime.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.Slime.html#run/1","title":"Still.Preprocessor.Slime.run/1","type":"function"},{"doc":"Still.Preprocessor.Renderer implementation for Slime files.","ref":"Still.Preprocessor.Slime.Renderer.html","title":"Still.Preprocessor.Slime.Renderer","type":"module"},{"doc":"","ref":"Still.Preprocessor.Slime.Renderer.html#create/1","title":"Still.Preprocessor.Slime.Renderer.create/1","type":"function"},{"doc":"Creates a hash based on the content of the file and generates a fingerprint to replace the output file.","ref":"Still.Preprocessor.URLFingerprinting.html","title":"Still.Preprocessor.URLFingerprinting","type":"module"},{"doc":"","ref":"Still.Preprocessor.URLFingerprinting.html#enabled?/0","title":"Still.Preprocessor.URLFingerprinting.enabled?/0","type":"function"},{"doc":"Returns the extension for the current file.This function can be overridden.","ref":"Still.Preprocessor.URLFingerprinting.html#extension/1","title":"Still.Preprocessor.URLFingerprinting.extension/1","type":"function"},{"doc":"Sets the extension for the current file and calls the render/1 function.","ref":"Still.Preprocessor.URLFingerprinting.html#run/1","title":"Still.Preprocessor.URLFingerprinting.run/1","type":"function"},{"doc":"Acts as a proxy between browser connections and file subscriptions. Whenever a relevant file changes, the browser connections are sent a reload message. Should only run in the dev environment.","ref":"Still.Web.BrowserSubscriptions.html","title":"Still.Web.BrowserSubscriptions","type":"module"},{"doc":"Adds a new browser connection to the subscribers list.","ref":"Still.Web.BrowserSubscriptions.html#add/1","title":"Still.Web.BrowserSubscriptions.add/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Still.Web.BrowserSubscriptions.html#child_spec/1","title":"Still.Web.BrowserSubscriptions.child_spec/1","type":"function"},{"doc":"","ref":"Still.Web.BrowserSubscriptions.html#start_link/1","title":"Still.Web.BrowserSubscriptions.start_link/1","type":"function"},{"doc":"Prettifies compilation errors to be displayed in the browser. Should only run in the dev environment.","ref":"Still.Web.ErrorFormatter.html","title":"Still.Web.ErrorFormatter","type":"module"},{"doc":"","ref":"Still.Web.ErrorFormatter.html#format/1","title":"Still.Web.ErrorFormatter.format/1","type":"function"},{"doc":"","ref":"Still.Compiler.PreprocessorError.html","title":"Still.Compiler.PreprocessorError","type":"exception"},{"doc":"Getting StartedStill is a simple static site generator for pragmatic developers. It's a modern tool that doesn't magically solve every single problem, but also doesn't create any new ones or generate new developer positions. It's a modern and sane experience, designed to be composable, easy to use and to grow in complexity with your needs and abilities.🚧 This documentation is still incomplete, but we built the landing page with Still to showcase some of its features. Have a look there if you can't find what you're looking for here.","ref":"getting_started.html","title":"Getting Started","type":"extras"},{"doc":"Still is in early active development. It requires Elixir 1.10.4 and Erlang 23.0.3. It may work with previous versions, but these are the versions we are using at the moment.mix archive.install hex still mix still.new mysiteThis will create an Elixir project in the folder mysite. You'll find an index.slime file inside priv/site.Run mix still.dev to start the development server. Then open http://localhost:3000 in your web browser to see the new website.","ref":"getting_started.html#quick-start","title":"Getting Started - Quick start","type":"extras"},{"doc":"Still can be used by itself or as part of another project.For new projectsRun mix archive.install hex still to install the archive on your system.Afterwards, create a site by running mix still.new my_site. That's it!Adding to an existing projectAdd still as a dependency in mix.exs:def deps do [ {:still, &quot;~&gt; 0.2.0&quot;} ] endOpen up config.exs and set the input and output folders:config :still, input: Path.join(Path.dirname(__DIR__), &quot;priv/site&quot;), output: Path.join(Path.dirname(__DIR__), &quot;_site&quot;)Finally, create a file index.slime in the input folder.","ref":"getting_started.html#installation","title":"Getting Started - Installation","type":"extras"},{"doc":"Run mix still.dev to start the development server. Then your website will be available in http://localhost:3000.Still is watching your file system for changes and it refreshes the browser when necessary. If there are any errors building the website, they will show up on the browser, along with the stack trace and the context where the error happen.If you run iex -S mix still.dev you'll get an interactive shell where you can test things quickly, such as API calls.","ref":"getting_started.html#development","title":"Getting Started - Development","type":"extras"},{"doc":"Run mix still.compile to compile the site to the output folder, which is _site by default. See the source code for the Github Action that deploys the landing page for more information on how to set up automatic deploys.","ref":"getting_started.html#production","title":"Getting Started - Production","type":"extras"},{"doc":"TemplateWe call template to almost every file in the input folder. These templates are transformed, and combined, to create the website.","ref":"templates.html","title":"Template","type":"extras"},{"doc":"In templates, you can usually write Elixir, which allows you to extend your site with abstractions to generate CSS or make API calls. Still comes with some functions out of the box to help you build your website. We call these view helpers.Including other filesUse the include/3 function to import the contents of another file. For instance, the following template includes the file _includes/head.slime:html head = include &quot;_includes/head.slime&quot; body = childrenBehind the scenes, Still will render head.slime to HTML and replace the function call with its contents.Linking to other filesThe link/2 function creates HTML hyperlinks. It takes care of specifying the rel and target when necessary, and it supports both full URLs or paths relative to the input folder. You should always use the link function to create hyperlinks between files, otherwise deploys to subfolders will not work. For instance:= link &quot;Home&quot;, to: &quot;/&quot; = link &quot;Blog&quot;, to: &quot;https://example.org&quot;Including imagesImage support is still (get it?) rudimentary: it can only generate images of different sizes. By default, it relies on ImageMagick, but if you add imageflow_ex as a dependency it will use that. The reason we do not use imageflow_ex by default is that it requires Rust. If you want to use ImageMagick, you also need to have it installed in your system.To insert an image use the function responsive_image/2. The first argument is the image's path and the second a keyword list. To transform the image we only care about the :sizes key, everything else will be rendered as an attribute on the generated img tag.The :sizes key is optional and by default we take the image's width to generate 3 smaller images. If you specify an array of integers, it will use that instead.A responsive image can be generated like this:&lt;%= responsive_image(&quot;example.jpg&quot;) %&gt;= responsive_image(&quot;cover.jpg&quot;, class: &quot;cover&quot;)Custom helpersYou can call any module from the templates (Enum.reverse(list) or MyHelpers.func()). However, if you want to write custom view helpers, you can add them to the configuration and their functions will be imported to the templates:config :still, view_helpers: [Your.Module]","ref":"templates.html#helpers","title":"Template - Helpers","type":"extras"},{"doc":"Layouts are templates that wrap other content. To use a layout, set the layout key in front matter. For instance:--- layout: _layout.slime --- h1 About PageThis will look for a _layout.slime in your input folder.The layout file must render the @children variable:doctype html html body = @childrenNotice that _layout.slime starts with an underscore. This is necessary since we don't want to compile the layout file by itself. Any file starting with an underscore isn't compiled to the output, but can be imported by other files.More ExamplesYou can use any templating language in your layout - it doesn't need to match the language of the content being wrapped. For instance, we can have a markdown file with a JavaScript wrapper:--- permalink: index.js layout: _console_layout.js --- # This is an h1And in _console_layout.js we have to render the children:console.log(&quot;&lt;%= @chidlren %&gt;&quot;);Layout ChainingAny file can include a layout, even files that work as layouts. For instance, you can have a blog post that uses a layout:--- layout: _post_layout.slime title: My blog post --- This is a blog post.And that layout can use another layout:--- layout: _layout.slime --- h1 = @title main = @childrenWhich may look something like this:html body div = @childrenThis allows you to cascade down and combine any templating language.","ref":"templates.html#layouts","title":"Template - Layouts","type":"extras"},{"doc":"Collections allow you to group file using a tags property:--- tags: post title: A blog post ---This file will be assigned to the collection post. Every file that specifies the post tag will be listed in the post collection. You can then iterate over this list by accessing the collections variable:ul = Enum.map Map.get(collections, &quot;post&quot;, []), fn x -&gt; li = link x[:title], to: x[:permalink]Collections are automatically updated when the files change. Files that use a collection will also be compiled whenever a file in a collection changes.","ref":"templates.html#collections","title":"Template - Collections","type":"extras"},{"doc":"Configuration","ref":"configuration.html","title":"Configuration","type":"extras"},{"doc":"Some files you don't need to transform, only to copy from the input to the output. That's what pass through copy is for.In your configuration files, if you specify a string on the pass_through_copy key, any file, or folder, whose relative path starts with that same string will be copied over to the output.config :still, pass_through_copy: [&quot;img/logo.png&quot;]In the example above, the file logo.png inside the img folder will be copied to the img folder in the output. But if you write something like this:config :still, pass_through_copy: [&quot;img&quot;]Any file or folder that starts with the string img will be copied, which may include an img folder or a file named img.png. So you need to be mindful of that.You can also use regular expressions:config still, pass_through_copy: [~r/.*\\.jpe?g/]The example above will will copy any file with a .jpg or .jpeg extension.Sometimes you want to alter the file name or path but keep the content of the files. The :pass_through_copy option allows this by using tuples. The key will be used to match the input folder, and the value will be used to transform the input path:config :still, pass_through_copy: [css: &quot;styles&quot;] # this is also valid: # config :still, # pass_through_copy: [{&quot;css&quot;, &quot;styles&quot;}]In the example above, the css folder from the input folder but will be renamed to styles in the output folder.","ref":"configuration.html#passthrough-copy","title":"Configuration - Passthrough copy","type":"extras"},{"doc":"PreprocessorsPreprocessors are the cornerstone of Still. A preprocessor chain can take a markdown file, execute its embedded Elixir, extract metadata from its front matter, transform it into HTML and wrap it in a layout.The default preprocessors are in the Preprocessor module. Each file in the input folder with one of the declared extensions is transformed and placed in the same relative place in the output folder. For instance, the file about.md will become about.html in the output folder, and a file inside a folder blog/post_1.md will also be in the same folder on the output directory blog/post_1.html.Notice that many file types, such as markdown and CSS, run through EEx, which means you can use EEx syntax in those files. Here's an example of a CSS file that uses EEx interpolation:html, body { color: &lt;%= Colors.white() %&gt;; }Files, or folders, that start with an underscore are ignored by the compilation step. These files can be set to run through the pass-through copy, or used as layouts and partials for other files.Custom preprocessorsIf the default preprocessors are not enough, you can extend Still with your own. Take the following example:defmodule YourSite.JPEG do use Still.Preprocessor @impl true def extension(_), do: &quot;.jpeg&quot; @impl true def render(file) do file end endA custom preprocessor is simply a module that calls use Still.Preprocessor and implements the render/2 and extension/1 functions. In this example, the render function is used to transform the content and the metadata of a file, and the extension function is used to set the resulting content type. This extension function is not mandatory.Preprocessors are always part of a transformation chain and each file will run through the chain, using the output of the one preprocessor as the input of the next. At the moment, the default transformation chains look like this:@default_preprocessors %{ &quot;.slim&quot; =&gt; [Frontmatter, Slime, OutputPath], &quot;.slime&quot; =&gt; [Frontmatter, Slime, OutputPath], &quot;.eex&quot; =&gt; [Frontmatter, EEx, OutputPath], &quot;.css&quot; =&gt; [EEx, CSSMinify, OutputPath, URLFingerprinting], &quot;.js&quot; =&gt; [EEx, JS, OutputPath, URLFingerprinting], &quot;.md&quot; =&gt; [Frontmatter, EEx, Markdown, OutputPath] }You can see that some files go through a front matter preprocessor, and CSS goes through EEx, which allows for the interpolation mentioned above.For the example preprocessor defined above, we can add it to the list in the configuration file:config :still, preprocessors: %{ &quot;.jpeg&quot; =&gt; [YourSite.JPEG] }This preprocessor doesn't do anything to the contents of a file, so the file on the output folder will look exactly like the file in the input folder.If you want to add a custom preprocessor to one of the default extensions, you need to redefine the whole pipeline. For example, if you want to add your own preprocessor for .css files but keep the existing ones do the following:config :still, preprocessors: %{ &quot;.css&quot; =&gt; [EEx, MyPreProcessor, CSSMinify, OutputPath, URLFingerprinting] }The preprocessors will be executed in the order you configure them.If you want to add a custom preprocessor to one of the default extensions, you need to redefine the whole pipeline. For example, if you want to add your own preprocessor for .css files but keep the existing ones do the following:config :still, preprocessors: %{ &quot;.css&quot; =&gt; [EEx, MyPreProcessor, CSSMinify, OutputPath, URLFingerprinting] }","ref":"preprocessors.html","title":"Preprocessors","type":"extras"}]